#include "adaptive_metropolis.hpp"


template <>
void AdaptiveMetropolis<double, double>::init()
{
    std::cout << "AdaptiveMetropolis<double, double>::init()" << std::endl;
    sigma_n = 1.0;
    mean_n = 0.0;
    sum_sq_val = 0.0;
    sum_val = 0.0;
}

template <>
void AdaptiveMetropolis<double, double>::update_sigma(double curr)
{
    niter += 1;
    
    mean_n = (mean_n * (niter - 1) + curr) / niter;
    sum_val += curr;
    sum_sq_val += curr * curr;

    std::cout << "update" << std::endl;
    std::cout << "mean_n: " << mean_n << ", sum_val: " << sum_val
              << ", sum_sq_val: " << sum_sq_val << std::endl;
    
    if (niter >= 2) {
        sigma_n = (sum_sq_val - 2 * mean_n * sum_val + mean_n * mean_n) / niter;
    } else {
        sigma_n = 1.0;
    }
    std::cout << "sigma_n: " << sigma_n << std::endl;
}

template <>
double AdaptiveMetropolis<double, double>::propose(double curr)
{   
    std::cout << "propose" << std::endl;
    double var, out;
    if (uniform_rng(0, 1, Rng::Instance().get()) < beta)
        var = 0.1 * 0.1;
    else
        var = 2.38 * 2.38 * sigma_n;

    std::cout << "sigma_n: " << sigma_n << std::endl;
    std::cout << "var " << var << std::endl;
    out = normal_rng(curr, sqrt(var), Rng::Instance().get());
    update_sigma(out);

    return out;
}

template <>
void AdaptiveMetropolis<MatrixXd, VectorXd>::init()
{
    sigma_n = MatrixXd::Identity(dim, dim);
    mean_n = VectorXd::Zero(dim);
    sum_sq_val = VectorXd::Zero(dim);
    sum_val = VectorXd::Zero(dim);
}

template <>
void AdaptiveMetropolis<MatrixXd, VectorXd>::update_sigma(VectorXd curr)
{
    niter += 1;
    mean_n = (mean_n * (niter - 1) + curr).array() / niter;
    sum_val += curr;

    sum_sq_val = curr * curr.transpose();
    sigma_n = (sum_sq_val - 2 * mean_n * sum_val + mean_n * mean_n).array() / niter;
}

template <>
VectorXd AdaptiveMetropolis<MatrixXd, VectorXd>::propose(VectorXd curr)
{
    MatrixXd var;
    VectorXd out;
    if (uniform_rng(0, 1, Rng::Instance().get()) < beta)
        var = MatrixXd::Identity(dim, dim) * 0.1 * 0.1 / dim;
    else
        var = 2.38 * 2.38 * sigma_n;

    out = multi_normal_rng(curr, var, Rng::Instance().get());
    update_sigma(out);

    return out;
}